@@ -1,8 +1,9 @@
 /*
   fastcluster: Fast hierarchical clustering routines for R and Python
 
-  Copyright © 2011 Daniel Müllner
-  <http://danifold.net>
+  Copyright:
+    * Until package version 1.1.23: © 2011 Daniel Müllner <http://danifold.net>
+    * All changes from version 1.1.24 on: © Google Inc. <http://google.com>
 
   This library implements various fast algorithms for hierarchical,
   agglomerative clustering methods:
@@ -35,22 +36,32 @@
   exception if a NaN distance value occurs.
 */
 
-#include <cstddef> // for std::ptrdiff_t
-#include <limits> // for std::numeric_limits<...>::infinity()
-#include <algorithm> // for std::fill_n
-#include <stdexcept> // for std::runtime_error
-#include <string> // for std::string
-
-// Microsoft Visual Studio does not have fenv.h
+// Older versions of Microsoft Visual Studio do not have the fenv header.
 #ifdef _MSC_VER
 #if (_MSC_VER == 1500 || _MSC_VER == 1600)
 #define NO_INCLUDE_FENV
 #endif
 #endif
-#ifndef NO_INCLUDE_FENV
+// NaN detection via fenv might not work on systems with software
+// floating-point emulation (bug report for Debian armel).
+#ifdef __SOFTFP__
+#define NO_INCLUDE_FENV
+#endif
+#ifdef NO_INCLUDE_FENV
+#pragma message("Do not use fenv header.")
+#else
+#pragma message("Use fenv header. If there is a warning about unknown #pragma STDC FENV_ACCESS, this can be ignored.")
+#pragma STDC FENV_ACCESS on
 #include <fenv.h>
 #endif
 
+#include <cmath> // for std::pow, std::sqrt
+#include <cstddef> // for std::ptrdiff_t
+#include <limits> // for std::numeric_limits<...>::infinity()
+#include <algorithm> // for std::fill_n
+#include <stdexcept> // for std::runtime_error
+#include <string> // for std::string
+
 #include <cfloat> // also for DBL_MAX, DBL_MIN
 #ifndef DBL_MANT_DIG
 #error The constant DBL_MANT_DIG could not be defined.
@@ -68,8 +79,31 @@
 #endif
 
 #ifndef INT32_MAX
+#ifdef _MSC_VER
+#if _MSC_VER >= 1600
 #define __STDC_LIMIT_MACROS
 #include <stdint.h>
+#else
+typedef __int32 int_fast32_t;
+typedef __int64 int64_t;
+#endif
+#else
+#define __STDC_LIMIT_MACROS
+#include <stdint.h>
+#endif
+#endif
+
+#define FILL_N std::fill_n
+#ifdef _MSC_VER
+#if _MSC_VER < 1600
+#undef FILL_N
+#define FILL_N stdext::unchecked_fill_n
+#endif
+#endif
+
+// Suppress warnings about (potentially) uninitialized variables.
+#ifdef _MSC_VER
+	#pragma warning (disable:4700)
 #endif
 
 #ifndef HAVE_DIAGNOSTIC
@@ -140,14 +174,6 @@ enum method_codes_vector {
   MAX_METHOD_VECTOR_CODE       = 3
 };
 
-enum {
-   // Return values
-  RET_SUCCESS        = 0,
-  RET_MEMORY_ERROR   = 1,
-  RET_STL_ERROR      = 2,
-  RET_UNKNOWN_ERROR  = 3
- };
-
 // self-destructing array pointer
 template <typename type>
 class auto_array_ptr{
@@ -167,7 +193,7 @@ public:
   auto_array_ptr(index const size, value const val)
     : ptr(new type[size])
   {
-    std::fill_n(ptr, size, val);
+    FILL_N(ptr, size, val);
   }
   ~auto_array_ptr() {
     delete [] ptr; }
@@ -182,7 +208,7 @@ public:
   template <typename index, typename value>
   void init(index const size, value const val) {
     init(size);
-    std::fill_n(ptr, size, val);
+    FILL_N(ptr, size, val);
   }
   inline operator type *() const { return ptr; }
 };
@@ -190,18 +216,12 @@ public:
 struct node {
   t_index node1, node2;
   t_float dist;
-
-  /*
-  inline bool operator< (const node a) const {
-    return this->dist < a.dist;
-  }
-  */
-
-  inline friend bool operator< (const node a, const node b) {
-    return (a.dist < b.dist);
-  }
 };
 
+inline bool operator< (const node a, const node b) {
+  return (a.dist < b.dist);
+}
+
 class cluster_result {
 private:
   auto_array_ptr<node> Z;
@@ -227,7 +247,7 @@ public:
 
   void sqrt() const {
     for (node * ZZ=Z; ZZ!=Z+pos; ++ZZ) {
-      ZZ->dist = ::sqrt(ZZ->dist);
+      ZZ->dist = std::sqrt(ZZ->dist);
     }
   }
 
@@ -237,14 +257,14 @@ public:
 
   void sqrtdouble(const t_float) const { // ignore the argument
     for (node * ZZ=Z; ZZ!=Z+pos; ++ZZ) {
-      ZZ->dist = ::sqrt(2*ZZ->dist);
+      ZZ->dist = std::sqrt(2*ZZ->dist);
     }
   }
 
   #ifdef R_pow
   #define my_pow R_pow
   #else
-  #define my_pow pow
+  #define my_pow std::pow
   #endif
 
   void power(const t_float p) const {
